// Worker class
// Assignment 2
// CSC375 - Lea
// Jacob Peck - 20111024

import java.util.Random;
import java.util.Map;

public class Worker {
  private final Random rng = new Random();
  private final Map<String,String> dataStore;
  private final boolean ignoreRWL;
  private final ReadWriteLock lock; // not used if ignoreRWL == true
  
  // the code below is taken (and modified) from 
  // http://stackoverflow.com/questions/41107/how-to-generate-a-random-alpha-numeric-string-in-java
  private static final char[] symbols = new char[36];
  static {
    for (int idx = 0; idx < 10; ++idx)
      symbols[idx] = (char) ('0' + idx);
    for (int idx = 10; idx < 36; ++idx)
      symbols[idx] = (char) ('a' + idx - 10);
  }

  private char[] buf;

  public String nextString(int length) {
    buf = new char[length];
    for (int idx = 0; idx < buf.length; ++idx) 
      buf[idx] = symbols[rng.nextInt(symbols.length)];
    return new String(buf);
  }
  // end random string generation code
  
  
  public Worker(boolean ignoreRWL, Map<String,String> dataStore, ReadWriteLock lock) {
    this.ignoreRWL = ignoreRWL;
    this.dataStore = dataStore;
    this.lock = lock;
  }
  
  public boolean doSomething() throws InterruptedException {
    float val = rng.nextFloat();
    if(val <= .9) return fakeAuth();  // 90% reads
    else return fakeRegister();       // 10% writes
  }
  
  private boolean fakeAuth() throws InterruptedException {
    String username = nextString(rng.nextInt(30)+1); // random username
    String password = nextString(rng.nextInt(30)+1); // random password
    //System.out.println("Authing... " + username + " ==> " + password);
    String pw = null;
    if (!ignoreRWL) {
      lock.readLock();
      pw = dataStore.get(username);
      lock.unlockRead();
    } else pw = dataStore.get(username);
    if (pw == null) return false;
    else return rot13(pw).equals(password); // unecrypt and check for equality
  }
  
  private boolean fakeRegister() throws InterruptedException {
    String username = nextString(rng.nextInt(30)+1); // random username
    String password = rot13(nextString(rng.nextInt(30)+1)); // random encrypted password
    //System.out.println("Registering... " + username + " ==> " + password);
    if (!ignoreRWL) {
      lock.writeLock();
      dataStore.put(username,password);
      lock.unlockWrite();
    } else dataStore.put(username,password);
    return true;
  }
  
  // inspired by http://introcs.cs.princeton.edu/java/31datatype/Rot13.java.html
  private String rot13(String input) {
    StringBuilder output = new StringBuilder();
    for (int i = 0; i < input.length(); i++) {
      char c = input.charAt(i);
      if (c >= 'a' && c <= 'm') c += 13;
      else if  (c >= 'n' && c <= 'z') c -= 13;
      else if  (c >= 'A' && c <= 'M') c += 13;
      else if  (c >= 'A' && c <= 'Z') c -= 13;
      output.append(c);
    }
    return output.toString();
  }
}
    
